//+------------------------------------------------------------------+
//|                                                      Test EA.mq4 |
//|                      Copyright  2009, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2009, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net"

#include <OrderReliable.mqh>

extern bool      varUseMM=false;
extern double    varLot=0.1;
extern double    varRisk=1.0;
extern int       varSlippage=30;
extern double    varAcctCap=1000000000; //1MM
extern double    varMASlope=30;
extern double    varFreezeZero=30; //if MarketInfo(Symbol(),MODE_FREEZELEVEL=0
extern double    varStopZero=30;  //if MarketInfo(Symbol(),MODE_STOPLEVEL=0
extern bool      varLogvalue=true;

double varOrderMagicNumber,varAcctBalance;
string varOrderComment;
int handle;


//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//----
   varOrderMagicNumber=funcGenerateMagicNumber(Period());
   varOrderComment=StringConcatenate(Symbol()," ",funcTimeFrame_Val2String(Period())); 
   if (varLogvalue)
   {
      handle=FileOpen("Test EA.csv",FILE_CSV|FILE_READ|FILE_WRITE,',');
      FileWrite(handle,"varCurtime","varFastMACurrent","varFastMALast","varTmpSlope","varSlowMACurrent","varSlowMALast","varSAR");
   }   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{
   int varOrdersTotal=OrdersTotal(); //used for ceiling stuff.
   double varAcctBalance=MathMin(AccountBalance(),MathMin(AccountEquity(),AccountFreeMargin()));   
   if (varOrdersTotal==0 && varAcctBalance>=varAcctCap) {return(0);} //no orders and we've reached our pre-set cap.
   bool varNewCandle=funcIsNewBar(Period());
   bool varNoOrder=true;
   if (varUseMM) {varLot=funcGetLots(varAcctBalance);}

   for (int i=0; i<OrdersTotal(); i++)
   {//1 +cycle by orders search
      if (OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if (OrderMagicNumber()!=varOrderMagicNumber) continue;
      int varOrderType=OrderType();
      varNoOrder=funcOrderWork(varOrderType);
      if (!varNoOrder && varNewCandle) {procMoveSL(varOrderType);}
   }
   
   if (varNoOrder && varAcctBalance<varAcctCap) 
   {
      string varSignal1=funcOpenSignal();
      if (varSignal1=="Long") {procOpenOrder(OP_BUY,varLot);}
      if (varSignal1=="Short") {procOpenOrder(OP_SELL,varLot);}
   }
   return(0);
}

bool funcOrderWork(int varOrderType)
{
      //moving SL.  make sure SL doesn't widen.  see chart.
      int timeframe=Period();
      bool varNoOrderTemp=false;
      double varFastMACurrent=iMA(Symbol(),timeframe,5,0,MODE_SMA,PRICE_TYPICAL,1);
      double varFastMALast=iMA(Symbol(),timeframe,5,0,MODE_SMA,PRICE_TYPICAL,2);
      double varTmpSlope=MathAbs(varFastMACurrent-varFastMALast)/Point;
      double varSAR=iSAR(Symbol(),timeframe,0.02,0.2,1);
      if (varOrderType==OP_BUY)
      {      
         if (varTmpSlope<varMASlope || varSAR>High[1])  
         {
            varNoOrderTemp=OrderCloseReliable(OrderTicket(),OrderLots(),Bid,varSlippage,Yellow);
         }
      }
      if (varOrderType==OP_SELL)
      {         
         if (varTmpSlope<varMASlope || varSAR<Low[1])
         {
            varNoOrderTemp=OrderCloseReliable(OrderTicket(),OrderLots(),Ask,varSlippage,Yellow);
         }        
      }
      return(varNoOrderTemp);
}

string funcOpenSignal()
{
      int timeframe=Period();
      double varFastMACurrent=iMA(Symbol(),timeframe,5,0,MODE_SMA,PRICE_TYPICAL,1);
      double varFastMALast=iMA(Symbol(),timeframe,5,0,MODE_SMA,PRICE_TYPICAL,2);
      double varTmpSlope=MathAbs(varFastMACurrent-varFastMALast)/Point;
      double varSlowMACurrent=iMA(Symbol(),timeframe,30,0,MODE_SMA,PRICE_TYPICAL,1);
      double varSlowMALast=iMA(Symbol(),timeframe,30,0,MODE_SMA,PRICE_TYPICAL,2);
      double varSAR=iSAR(Symbol(),timeframe,0.02,0.2,1);
      double varSARNow=iSAR(Symbol(),timeframe,0.02,0.2,0);
      if (varSlowMACurrent>Low[1] && varSlowMACurrent<High[1]) {return("nothing");}
      if (varFastMACurrent>varFastMALast && varSlowMACurrent>varSlowMALast && varFastMACurrent>varSlowMACurrent && 
          varSAR<Low[1] && varTmpSlope>varMASlope && varSARNow<Low[0]) 
         {return("Long");}
      if (varFastMACurrent<varFastMALast && varSlowMACurrent<varSlowMALast && varFastMACurrent<varSlowMACurrent && 
          varSAR>High[1] && varTmpSlope>varMASlope && varSARNow>High[0])
         {return("Short");}
      return("nothing");
}

void procOpenOrder(int varOrderType,double varLots)
{
   double varTP=0; int varTmpTicket;
   double varSL=funcSL(varOrderType);
   varSL=funcSL(varOrderType);
   varTP=funcTP(varOrderType);
     
   if (varLots==0) 
   {
      Print("Not enought money in Account");
      return(0);
   }
   if (varOrderType==OP_BUY)
   {
      varTmpTicket=OrderSendReliable(Symbol(),OP_BUY,varLots,Ask,varSlippage,varSL,varTP,varOrderComment,varOrderMagicNumber,0,Blue);              
   }
   if (varOrderType==OP_SELL)
   {
      varTmpTicket=OrderSendReliable(Symbol(),OP_SELL,varLots,Bid,varSlippage,varSL,varTP,varOrderComment,varOrderMagicNumber,0,Red);  
   }
   procLogIt();
}

void procLogIt()
{
      int timeframe=Period();
      double varFastMACurrent=iMA(Symbol(),timeframe,5,0,MODE_SMA,PRICE_TYPICAL,1);
      double varFastMALast=iMA(Symbol(),timeframe,5,0,MODE_SMA,PRICE_TYPICAL,2);
      double varTmpSlope=MathAbs(varFastMACurrent-varFastMALast)/Point;
      double varSlowMACurrent=iMA(Symbol(),timeframe,30,0,MODE_SMA,PRICE_TYPICAL,1);
      double varSlowMALast=iMA(Symbol(),timeframe,30,0,MODE_SMA,PRICE_TYPICAL,2);
      double varSAR=iSAR(Symbol(),timeframe,0.02,0.2,1);
      datetime varcurtime=iTime(Symbol(),PERIOD_M1,0);   
   FileWrite(handle,varTmpSlope,MathAbs(varSlowMACurrent-varSlowMALast));
}
double funcSL(int varOrderType)
{
   int timeframe=Period();
   double varSL=iSAR(Symbol(),timeframe,0.02,0.2,1);
/*   if (varOrderType==OP_BUY)
   {
        varSL=Low[1];
   }
   if (varOrderType==OP_SELL)
   {
        varSL=High[1];
   }*/
   return(varSL);
}

double funcTP(int varOrderType)
{
   double varTP=0; 
   double varFreezeLevel=MarketInfo(Symbol(),MODE_FREEZELEVEL);
   double varStopLevel=MarketInfo(Symbol(),MODE_STOPLEVEL);
   double varSpread=MarketInfo(Symbol(),MODE_SPREAD);
   if (varFreezeLevel==0) {varFreezeLevel=varFreezeZero;}
   if (varStopLevel==0) {varStopLevel=varStopZero;}
   if (varOrderType==OP_BUY)
   {
      varTP=((varFreezeLevel+varSpread)*Point)+Ask;
   }
   if (varOrderType==OP_SELL)
   {
      varTP=Bid-((varFreezeLevel+varSpread)*Point);
   }
   ///return(varTP);
   return(0);
}

void procMoveSL(int varOrderType)
{
   int timeframe=Period();
   double varSAR=iSAR(Symbol(),timeframe,0.02,0.2,1);   
   if (varOrderType==OP_BUY)
   {
      if (varSAR>OrderStopLoss())
      {
         OrderModifyReliable(OrderTicket(),OrderOpenPrice(),varSAR,OrderTakeProfit(),0,Yellow);
      }
   }
   if (varOrderType==OP_SELL)
   {  
      if (varSAR<OrderStopLoss())
      {
         OrderModifyReliable(OrderTicket(),OrderOpenPrice(),varSAR,OrderTakeProfit(),0,Yellow);
      }
   }     
}

double funcGetLots(double varAcctBalance)
{
   if (!varUseMM) {return(varLot);}
   double varMinLot=MarketInfo(Symbol(),MODE_MINLOT);
   double varMaxLot=MarketInfo(Symbol(),MODE_MAXLOT);
   double varLot=NormalizeDouble((varAcctBalance*(varRisk/100))*AccountLeverage()/MarketInfo(Symbol(),MODE_LOTSIZE),2);
   if (varLot>varMaxLot) return(varMaxLot);
   if (varLot<varMinLot) return(0);
   return(varLot);
}


//+================== Support functions =============================+
int funcGenerateMagicNumber(int vartempTimeframe)
{
   int vartempTimeframeval=funcTimeFrame_Const2Val(vartempTimeframe);
   int vartempSymbol=funcSymbol2Val(Symbol());
   return(vartempTimeframeval+vartempSymbol);    
}

bool funcIsNewBar(int timeFrame)
   {
   bool res=false;
   
   // the array contains open time of the current (zero) bar
   // for 7 (seven) timeframes
   static datetime _sTime[7];  
   int i=6;
   //Note: i below will be 6 or timeframe will be day.
   switch (timeFrame) 
      {
      case 1  : i=0; break;
      case 5  : i=2; break;
      case 15 : i=3; break;
      case 30 : i=4; break;
      case 60 : i=5; break;
      case 240: break;
      case 1440:break;
      default:  timeFrame = 1440;
      }
//----
   if (_sTime[i]==0 || _sTime[i]!=iTime(Symbol(),timeFrame,0))
      {
      _sTime[i] = iTime(Symbol(),timeFrame,0);
      res=true;
      }
      
//----
   return(res);   
   }

//+------------------------------------------------------------------+
//| Time frame interval appropriation  function                      |
//+------------------------------------------------------------------+
int funcTimeFrame_Const2Val(int varConstant) 
{
   switch(varConstant) 
   {
      case 1:  
         return(1000);
      case 5:  
         return(2000);
      case 15:
         return(3000);
      case 30:
         return(4000);
      case 60:
         return(5000);
      case 240:
         return(6000);
      case 1440:
         return(7000);
      case 10080:
         return(8000);
      case 43200:
         return(9000);
   }
}
//+------------------------------------------------------------------+
//| Time frame string appropriation  function                               |
//+------------------------------------------------------------------+
string funcTimeFrame_Val2String(int varValue) 
{
   switch(varValue) 
   {
      case 1:  
         return("PERIOD_M1");
      case 2:  
         return("PERIOD_M5");
      case 3:
         return("PERIOD_M15");
      case 4:
         return("PERIOD_M30");
      case 5:
         return("PERIOD_H1");
      case 6:
         return("PERIOD_H4");
      case 7:
         return("PERIOD_D1");
      case 8:
         return("PERIOD_W1");
      case 9:
         return("PERIOD_MN1");
   	default: 
   		return("undefined " +varValue);
   }
}
int funcSymbol2Val(string varSymbol) 
{
   string varMySymbol = StringSubstr(varSymbol,0,6);
	if(varMySymbol=="AUDCAD" || varMySymbol=="AUDCADm") return(1);
	if(varMySymbol=="AUDJPY" || varMySymbol=="AUDJPYm") return(2);
	if(varMySymbol=="AUDNZD" || varMySymbol=="AUDNZDm") return(3);
	if(varMySymbol=="AUDUSD" || varMySymbol=="AUDUSDm") return(4);
	if(varMySymbol=="CHFJPY" || varMySymbol=="CHFJPYm") return(5);
	if(varMySymbol=="EURAUD" || varMySymbol=="EURAUDm") return(6);
	if(varMySymbol=="EURCAD" || varMySymbol=="EURCADm") return(7);
	if(varMySymbol=="EURCHF" || varMySymbol=="EURCHFm") return(8);
	if(varMySymbol=="EURGBP" || varMySymbol=="EURGBPm") return(9);
	if(varMySymbol=="EURJPY" || varMySymbol=="EURJPYm") return(10);
	if(varMySymbol=="EURUSD" || varMySymbol=="EURUSDm") return(11);
	if(varMySymbol=="GBPCHF" || varMySymbol=="GBPCHFm") return(12);
	if(varMySymbol=="GBPJPY" || varMySymbol=="GBPJPYm") return(13);
	if(varMySymbol=="GBPUSD" || varMySymbol=="GBPUSDm") return(14);
	if(varMySymbol=="NZDUSD" || varMySymbol=="NZDUSDm") return(15);
	if(varMySymbol=="USDCAD" || varMySymbol=="USDCADm") return(16);
	if(varMySymbol=="USDCHF" || varMySymbol=="USDCHFm") return(17);
	if(varMySymbol=="USDJPY" || varMySymbol=="USDJPYm") return(18);
	return(19);
}


